var api = require('../../restApi/api.js');
var config = require('../../config.js');
var uuid = require('../../utils/Math.uuid.js');

//获取应用实例
const app = getApp();
Page({

  /**
   * 页面的初始数据
   */
  data: {
    roomId: '',
    userName: '',

    scene: [{
      id: 1,
      value: '音视频',
      checked: true
    }, {
      id: 2,
      value: '白板',
      checked: false
    }],

    roomIdErrMsg: '',
    errorMessage: '',
    userId: uuid.Math.uuid(),
  },

  getAccessToken: function () {
    var self = this;
    var promise = new Promise((resolve, reject) => {
      api.getAccessToken(config.accessKey, config.secretKey).then((data) => {
        resolve(data.token);
      }).catch((error) => {
        reject(error);
      });
    });
    return promise;
  },

  //绑定webChatId输入框
  setRoomId: function (e) {
    var self = this;
    self.setData({
      roomId: e.detail.value
    });
  },

  //绑定webChatName输入框
  setUserName: function (e) {
    var self = this;
    self.setData({
      userName: e.detail.value
    });
  },

  radioChange: function (e) {
    console.log('+++touch.js radioChange(),radio发生change事件，携带value值为：', e.detail.value)
    const scene = this.data.scene
    for (let i = 0, len = scene.length; i < len; ++i) {
      scene[i].checked = scene[i].id == e.detail.value
    }
    this.setData({
       scene
    })
  },



  joinRoom: function () {
    console.log(new Date(), "+++touch.js joinRoom()...");
    var self = this;
    wx.showLoading({
      title: '加入会议...',
      mask: true
    });
    self.getAccessToken().then(function (token) {
      wx.hideLoading();
      console.log(new Date(), "+++touch.js joinRoom(),accessToken:", token);
      console.log(new Date(), "+++touch.js joinRoom(),app.data.config.serverUrl:", app.data.config.serverUrl);
      
      self.addRouteHandle(token);
    }).catch(function (error) {
      self.setData({
        errorMessage: '获取访问令牌失败。原因:' + error.message
      })
    });
  },

  addRouteHandle: function (token) {
    console.log(new Date(), "+++touch.js addRouteHandle()...");
    var self = this;

    var roomId = self.data.roomId;
    if (!roomId) {
      self.setData({
        roomIdErrMsg: "请输入会议号"
      })
      return;
    } else {
      self.setData({
        roomIdErrMsg: ""
      })
    }

    var userName = self.data.userName;
    if (!userName) {
      self.setData({
        roomIdErrMsg: "请输入用户名"
      })
      return;
    } else {
      self.setData({
        roomIdErrMsg: ""
      })
    }
    wx.showLoading({
      title: '会议初始化...',
      mask: true
    });
    app.data.userInfo.roomId = roomId;
    app.data.userInfo.userName = self.data.userName;
    app.data.getAvdSDK.avdEngine.init(config.serverUrl, token).then(self.initSuccess).catch(self.initError)
  },

  /**
   * 生命周期函数--监听页面加载
   */
  onLoad: function (options) {   
  },


  initSuccess: function () {
    console.log(new Date(), "+++touch.js initSuccess()");

    wx.showLoading({
      title: '初始化成功...',
      mask: true
    });
    app.data.getAvdSDK.room = app.data.getAvdSDK.avdEngine.obtainRoom(this.data.roomId);
    app.data.getAvdSDK.room.join(this.data.userId, this.data.userName, '', '').then(this.joinSuccess).catch(this.joinError);
  },

  initError: function (e) {
    console.log('initError',e);
    wx.showModal({
      title: '出错提示',
      content: "网络故障，服务器连接超时。可刷新重试! 错误码:" + e.error.code + ",错误信息：" + e.error.message,
      showCancel: false
    })
  },

  joinSuccess:function() {
    console.log(new Date(), "+++touch.js joinSuccess()");
    wx.showLoading({
      title: '正在跳转...',
      mask: true
    });


    var sceneId;
    this.data.scene.map((item, index) => {
      if (item.checked) {
        sceneId = item.id;
      }
    })
    var url = "../home/home";

    if(sceneId ==2){
       url = "../board/board";
      //  url = "../draw/draw";
    }
    wx.reLaunch({
      url: url
    });
  },

  joinError: function (error) {
    wx.hideLoading();
    console.log(new Date(), "+++touch.js joinError(),error:", error);
    wx.showModal({
      title: '出错提示',
      content: "加会失败了! 错误码:" + error.code + ",错误信息：" + error.message,
      showCancel: false
    })
  },


  /**
   * 生命周期函数--监听页面初次渲染完成
   */
  onReady: function () {

  },

  /**
   * 生命周期函数--监听页面显示
   */
  onShow: function () {

  },

  /**
   * 生命周期函数--监听页面隐藏
   */
  onHide: function () {

  },

  /**
   * 生命周期函数--监听页面卸载
   */
  onUnload: function () {

  },

  /**
   * 页面相关事件处理函数--监听用户下拉动作
   */
  onPullDownRefresh: function () {

  },

  /**
   * 页面上拉触底事件的处理函数
   */
  onReachBottom: function () {

  },

  /**
   * 用户点击右上角分享
   */
  onShareAppMessage: function () {

  }
})