var config = require('../config.js');
var protocolStr = "https:";

module.exports = {

  /**
   * 获取访问令牌
   * @param {String} access_key 
   * @param {String} secret_key 
   */
  getAccessToken:function (access_key, secret_key) {
      var serverUrl = config.serverUrl;
      console.log("++++getAccessToken(),serverUrl:", serverUrl);
      var sdkapiBaseURL = protocolStr + "//" + serverUrl + "/sdkapi/v2";
      console.log("++++getAccessToken restful URL:" + sdkapiBaseURL + "/app/get_token");

    return fetchGet(sdkapiBaseURL+'/app/get_token', { 
          "access_key": access_key, 
          "secret_key": secret_key 
        });
    },


  /**
   * 创建房间
   * @param {String} token-访问令牌
   * @param {String} topic-会议主题 
   * @param {Boolean} enable_auto_cloud_recording-是否开启云端自动录制，默认false
   * @param {String} user_id-房间创建者用户ID 
   * @param {String} room_id-指定房间ID，如果不传服务器生成随机号
   * @param {Boolean} enable_auto_avc-是否启用自动视频AVC模式（视频合屏），默认false
   */
  createRoom:function (token, topic, user_id, room_id, enable_auto_cloud_recording, enable_auto_avc) {
    var paramsObj = {
      "token": token,
      "topic": topic,
      "user_id": user_id,
    }

    if (room_id) {
      paramsObj.room_id = room_id;
    }

    if (enable_auto_cloud_recording) {
       paramsObj.enable_auto_cloud_recording = enable_auto_cloud_recording;
    }

    if (enable_auto_avc) {
        paramsObj.enable_auto_avc = enable_auto_avc;
    }

    var serverUrl = config.serverUrl;
    console.log("++++createRoom(),serverUrl:", serverUrl);
    var avd2apiBaseURL = protocolStr + "//" + serverUrl + "/sdkapi/v2"
    console.log("++++createRoom restful URL:" + avd2apiBaseURL + "/room/create");

    return fetchGet(avd2apiBaseURL + '/room/create', paramsObj);
  },





  /**
   * 更新实时验证令牌token和会话isession_id
   * module：system/系统模块, 错误码区间:10200
   */
  authRefresh: function(session_id, room_token) {
    var serverUrl = config.serverUrl;
    console.log("++++authRefresh(),serverUrl:", serverUrl);
    var avd2apiBaseURL = protocolStr + "//" + serverUrl + "/avd2api/v1"
    console.log("++++authRefresh restful URL:" + avd2apiBaseURL + "/auth/refresh");

    return fetchGet(avd2apiBaseURL + "/auth/refresh", {
      "session_id": session_id,
      "room_token": room_token
    });
  },


  /**
   * 登录
   * 访问URL: auth/login
   * 错误码区间:10100-10199
   */
  userLogin: function(email, password) {
    var serverUrl = config.serverUrl;
    console.log("++++userLogin(),serverUrl:", serverUrl);
    var avd2apiBaseURL = protocolStr + "//" + serverUrl + "/avd2api/v1"
    console.log("++++userLogin restful URL:" + avd2apiBaseURL + "/auth/login");
    return fetchGet(avd2apiBaseURL + "/auth/login", {
      "email": email,
      "password": password
    });
  },


  /**
   * 获取房间参数
   * 访问URL: /room/get_room_params
   * module：system/系统模块, 错误码区间:10200
   */
  getRoomParams: function(roomId, sessionId) {
    var serverUrl = config.serverUrl;
    console.log("++++getRoomParams(),serverUrl:", serverUrl);
    var avd2apiBaseURL = protocolStr + "//" + serverUrl + "/avd2api/v1"
    console.log("++++getRoomParams restful URL:" + avd2apiBaseURL + "/room/get_room_params");
    sessionId = sessionId ? sessionId : '';
    return fetchGet(avd2apiBaseURL + "/room/get_room_params", {
      "room_id": roomId,
      "session_id": sessionId
    });
  },

  /**
   * 游客获取房间token
   * 访问URL: auth/get_room_token
   * 错误码区间:10100-10199
   */
  getVisitoRoomToken: function(roomId, userName) {
    var serverUrl = config.serverUrl;
    console.log("++++getVisitoRoomToken(),serverUrl:", serverUrl);
    var avd2apiBaseURL = protocolStr + "//" + serverUrl + "/avd2api/v1"
    console.log("++++getVisitoRoomToken restful URL:" + avd2apiBaseURL + "/auth/get_room_token");
    return fetchGet(avd2apiBaseURL + "/auth/get_room_token", {
      "room_id": roomId,
      "name": userName
    });
  }
}

/**
 * 自定义get函数，返回Promise
 * @param { String } url 接口网址
 * @param { arrayObject } data 要传的数组对象 例如: { name: '武当山道士', age: 32 }
 * @return { Promise }    promise 返回promise供后续操作
 */
function fetchGet(url, data, thirdRsetful) {
  var promise = new Promise((resolve, reject) => {
    var that = this;
    var getData = data;

    /*
    //自动添加签名字段到getData，makeSign(obj)是一个自定义的生成签名字符串的函数
    getData.signature = that.makeSign(getData);
    */

    //网络请求
    wx.request({
      url: url,
      data: getData,
      method: 'GET',
      header: {
        'content-type': 'application/x-www-form-urlencoded'
      },
      success: function(res) {
        if (thirdRsetful) {
          if (res.data.errcode) {
            reject(res.data);
          } else {
            resolve(res.data);
          }
        } else {
          if (res.data.ret === 0) {
            resolve(res.data);
          } else {
            reject(res.data);
          } 
        }
      },
      error: function(e) {
        var error = {};
        if (e.response) {
          error = new Error(e.response.status, e.response.data);
        } else {
          error = new Error(10000, e.message);
        }
        reject(error);
      }
    })
  });
  return promise;
};


/**
 * 自定义post函数，返回Promise
 * @param { String } url 接口网址
 * @param { arrayObject } data 要传的数组对象 例如: { name: '武当山道士', age: 32 }
 * @return { Promise }    promise 返回promise供后续操作
 */
function fetchPost(url, data, thirdRsetful) {
  var promise = new Promise((resolve, reject) => {
    var that = this;
    var postData = data;

    /*
    //自动添加签名字段到postData，makeSign(obj)是一个自定义的生成签名字符串的函数
    postData.signature = that.makeSign(postData);
    */

    //网络请求
    wx.request({
      url: url,
      data: postData,
      method: 'POST',
      responseType: 'arraybuffer',
      header: {
        'content-type': 'application/json'
      },
      success: function(res) {
        if (thirdRsetful) {
          if (res.data.errcode) {
            reject(res.data);
          } else {
            resolve(res.data);
          }
        } else {
          if (res.data.ret === 0) {
            resolve(res.data);
          } else {
            reject(res.data);
          }
        }
      },
      error: function(e) {
        var error = {};
        if (e.response) {
          error = new Error(e.response.status, e.response.data);
        } else {
          error = new Error(10000, e.message);
        }
        reject(error);
      }
    })
  });
  return promise;
};

function Error(object, message) {
  this.error = {};

  if (typeof (object) == "object" && typeof (message) == 'undefined') {
    this.error.code = object.code;
    this.error.message = object.message;
  } else {
    this.error.code = object;
    this.error.message = message;
  }
}