import { config } from 'config'
var protocolStr = "https:";

module.exports = {

  /**
   * 获取访问令牌
   * @param {String} access_key 
   * @param {String} secret_key 
   */
  getAccessToken: function (access_key, secret_key) {
    var serverUrl = config.serverUrl;
    console.log(new Date(), "=======getAccessToken(),serverUrl:", serverUrl);
    var sdkapiBaseURL = protocolStr + "//" + serverUrl + "/sdkapi/v2";
    console.log(new Date(), "getAccessToken restful URL:" + sdkapiBaseURL + "/app/get_token");

    return fetchGet(sdkapiBaseURL + '/app/get_token', {
      "access_key": access_key,
      "secret_key": secret_key
    });
  },


  /**
   * 创建房间
   * @param {String} token-访问令牌
   * @param {String} topic-会议主题 
   * @param {String} user_id-房间创建者用户ID
   * @param {String} room_id-指定房间ID，如果不传服务器生成随机号
   * @param {Boolean} enable_auto_cloud_recording-是否开启云端自动录制，默认false
   * @param {Boolean} enable_auto_avc-是否启用自动视频AVC模式（视频合屏），默认false
   */
  createRoom: function (token, topic, user_id, room_id, enable_auto_cloud_recording, enable_auto_avc) {
    var paramsObj = {
      "token": token,
      "topic": topic,
      "user_id": user_id,
    }

    if (room_id) {
      paramsObj.room_id = room_id;
    }

    if (enable_auto_cloud_recording) {
      paramsObj.enable_auto_cloud_recording = enable_auto_cloud_recording;
    }

    if (enable_auto_avc) {
      paramsObj.enable_auto_avc = enable_auto_avc;
    }

    var serverUrl = config.serverUrl;
    console.log(new Date(), "=======createRoom(),serverUrl:", serverUrl);
    var avd2apiBaseURL = protocolStr + "//" + serverUrl + "/sdkapi/v2"
    console.log(new Date(), "createRoom restful URL:" + avd2apiBaseURL + "/room/create");

    return fetchGet(avd2apiBaseURL + '/room/create', paramsObj);
  },
}

/**
 * 自定义get函数，返回Promise
 * @param { String } url 接口网址
 * @param { arrayObject } data 要传的数组对象 例如: { name: '武当山道士', age: 32 }
 * @return { Promise }    promise 返回promise供后续操作
 */
function fetchGet(url, data) {
  var promise = new Promise((resolve, reject) => {
    var that = this;
    var getData = data;

    /*
    //自动添加签名字段到getData，makeSign(obj)是一个自定义的生成签名字符串的函数
    getData.signature = that.makeSign(getData);
    */

    //网络请求
    wx.request({
      url: url,
      data: getData,
      method: 'GET',
      header: {
        'content-type': 'application/x-www-form-urlencoded'
      },
      success: function (res) {
        if (res.data.ret === 0) {
          resolve(res.data);
        } else {
          var error = new Error(res.data.ret, res.data.msg);
          reject(error);
        }

      },
      fail: function (e) {
        console.log("==============fetchGet(),======e:", e);
        var error = {};
        if (e.errMsg) {
          error = new Error(505, e.errMsg);
        } else if (e.response) {
          error = new Error(e.response.status, e.response.data);
        } else {
          error = new Error(10000, e.message);
        }
        reject(error);
      }
    })
  });
  return promise;
};


/**
 * 自定义post函数，返回Promise
 * @param { String } url 接口网址
 * @param { arrayObject } data 要传的数组对象 例如: { name: '武当山道士', age: 32 }
 * @return { Promise }    promise 返回promise供后续操作
 */
function fetchPost(url, data) {
  var promise = new Promise((resolve, reject) => {
    var that = this;
    var postData = data;

    /*
    //自动添加签名字段到postData，makeSign(obj)是一个自定义的生成签名字符串的函数
    postData.signature = that.makeSign(postData);
    */

    //网络请求
    wx.request({
      url: url,
      data: postData,
      method: 'POST',
      responseType: 'arraybuffer',
      header: {
        'content-type': 'application/json'
      },
      success: function (res) {
        if (res.data.ret === 0) {
          resolve(res.data);
        } else {
          var error = new Error(res.data.ret, res.data.msg);
          reject(error);
        }
      },
      fail: function (e) {
        console.log("==============fetchPost(),======e:", e);
        var error = {};
        if (e.errMsg) {
          error = new Error(505, e.errMsg);
        } else if (e.response) {
          error = new Error(e.response.status, e.response.data);
        } else {
          error = new Error(10000, e.message);
        }
        reject(error);
      }
    })
  });
  return promise;
};

function Error(object, message) {
  if (typeof (object) == "object" && typeof (message) == 'undefined') {
    var e = {
      code: object.code,
      message: object.message
    }
    return e;
  } else {
    var e = {
      code: object,
      message: message
    }
    return e;

  }
}