
var api = require('../../restApi/api.js');

//获取应用实例
const app = getApp();

Page({

  /**
   * 页面的初始数据
   */
  data: {
    rpcManager: null,

    seatUserId: '', //ACD分配的可用座席ID

    seatUserName: '', //ACD分配的可用座席名

    tipsMessage: '',

    isJoinSucces: false,
    
    inviteState: 0, //状态， 0-邀请前，1-邀请中，2-邀请被接收，3-邀请被拒绝，4-邀请超时,5-取消呼叫
  },

  /**
   * 生命周期函数--监听页面加载
   */
  onLoad: function(options) {
    var self = this;
    self.registerClientCallback();

    self.data.rpcManager = app.data.avdSDK.client.getRpcManager();
    self.data.inviteManager = app.data.avdSDK.client.getInviteManager();
    
    self.sendRpcCall();
    self.sendAcdSetClientOptions();
  },


  /**
   * 注册Client级别回调 
   */
  registerClientCallback: function() {
    app.data.avdSDK.client.addCallback(app.data.avdSDK.Enum.ClientCallbackEnum.NATIVE_PAYLOAD_NOTIFY, this.onNativePayload);
    app.data.avdSDK.client.addCallback(app.data.avdSDK.Enum.ClientCallbackEnum.RPC_NOTIFICATION, this.onRpcNotification);
    app.data.avdSDK.client.addCallback(app.data.avdSDK.Enum.ClientCallbackEnum.MQTT_CONNECT_STATE, this.onMqttConnectState);
  },


  onNativePayload: function(topic, payload) {
    console.log(new Date(),"====================onNativePayload(),topic:" + topic + ",payload:", payload);
  },


  //MQTT 连接状态显示
  onMqttConnectState: function (state){
    console.log("=======================onMqttConnectState(),state:", state);
    if (state == app.data.avdSDK.Enum.mqttConnectStateEnum.connect) {
      wx.hideLoading();
    } else if (state == app.data.avdSDK.Enum.mqttConnectStateEnum.reconnect) {
      wx.showLoading({
        title: '路由重连中...',
        mask: true
      });
    } else {
      wx.hideLoading();
    }
  },



  //RPC通知回调
  onRpcNotification: function(data) {
    console.log(new Date(),"onRpcNotification(), data :", data);
    if (data.type == "error") { 
      var error  = {};
      error.code = data.error.code;
      error.message = data.error.message;
      this.setTipsMessageError(error);
    }else{
       this.setTipsMessage(data);
    }
  },
  
  toCall: function() {
    var self = this;

    if (this.inviteState5ToReturn("toCall()")) {
      return;
    };

    console.log(new Date(),"============tocall()...");
    self.createRoom().then(function(roomId) {
      console.log("========toCall(),roomId:", roomId);
      self.data.roomId = roomId;
      self.call();
    }).catch(function(error) {
      console.log(new Date(),"房间创建失败。error:", error);
      self.setData({
        tipsMessage: '房间创建失败。error:' + error.message
      })
    });
  },


  call: function() {
    var self = this;
    if (this.inviteState5ToReturn("call()")){
      return;
    };

    var options = {
      'roomId': self.data.roomId,
      'name': app.data.userInfo.webChatName
    };

    console.log(new Date(),"=============call(),app.data.avdSDK.client:", app.data.avdSDK.client);
    console.log(new Date(),"=============call(),app.data.avdSDK.client.selfUser:", app.data.avdSDK.client.selfUser);

    if (!app.data.avdSDK.client.selfUser){
        return ;
    }
    var userId = app.data.avdSDK.client.selfUser.userId;

    console.log(new Date(),"=============call(),userId:", userId);
    var callInvite = self.data.inviteManager.createInvite(userId, self.data.seatUserId, options);
    
    app.data.callInvite = callInvite;

    self.registerInviteCallback(callInvite);

    self.data.inviteManager.call(callInvite).then(function() {
      console.log(new Date(),'邀请发送成功!');
      self.setData({
        tipsMessage: "邀请发送成功,等待对方接受...",
        inviteState: 1
      });
    }).catch(function(error) {
      console.log(new Date(),'邀请发送失败！' + error.code + '/' + error.message);
      self.setData({
         tipsMessage: "邀请发送失败" + error.code + '/' + error.message,
      });
    });
  },

  inviteState5ToReturn: function(msg){
     var isReturn = false;
     if (this.data.inviteState == 5) {
        console.log(new Date(), "===WWWWWWWW===inviteState==5 to return. msg:", msg);
        isReturn = true;
     }
     return isReturn;
  },


  /**
   * 注册Invite级别回调 
   */
  registerInviteCallback: function(invite) {
    var self = this;

    invite.addCallback(app.data.avdSDK.Enum.InviteCallbackEnum.CALL_ACCEPT_NOTIFY, self.onCallAccept);
    invite.addCallback(app.data.avdSDK.Enum.InviteCallbackEnum.CALL_REJECT_NOTIFY, self.onCallReject);
    invite.addCallback(app.data.avdSDK.Enum.InviteCallbackEnum.CALL_TIMEOUT_NOTIFY, self.onCallTimeout);
    invite.addCallback(app.data.avdSDK.Enum.InviteCallbackEnum.CALL_RINGING_NOTIFY, self.onCallRinging);

    invite.addCallback(app.data.avdSDK.Enum.InviteCallbackEnum.CALL_CLOSE_NOTIFY, self.onCallClose);
  },

  //邀请请求已被接受的回调
  onCallAccept: function(toUserId, options) {
    var self = this;
    if (this.inviteState5ToReturn("onCallAccept()")) {
      return;
    };

    console.log(new Date(),"=====wwwwwwww=onCallAccept()=====toUserId:" + toUserId + ";options:" + options);
    this.setData({
      tipsMessage: self.data.seatUserName + "接受了您的邀请",
      inviteState: 2,
    });
  
    self.joinRoom();
  },

  //邀请请求已被拒绝的回调
  onCallReject: function(toUserId, reson) {
    var self = this;
    console.log(new Date(),"======onCallReject()=====toUserId:" + toUserId + ";reson:" + reson);
    self.setData({
      tipsMessage: self.data.seatUserName + "拒绝了邀请",
      inviteState: 3,
    })
  },


  //邀请请求无响应，超时回调
  onCallTimeout: function(toUserId) {
    var self = this;
    console.log(new Date(),"======onCallTimeout()=====toUserId:" + toUserId);
    self.setData({
      tipsMessage: "发送邀请给" + self.data.seatUserName + "无响应，已超时",
      inviteState: 4,
    })
  },


  //邀请请求响铃回调
  onCallRinging: function(toUserId) {
    console.log(new Date(),"======onCallRinging()=====toUserId:" + toUserId);
  },


  onCallClose: function(fromUserId, reson) {
     console.log(new Date(),"======onCallClose()=====fromUserId:" + fromUserId + ";reson:" + reson);
  },


  joinRoom: function() {
    console.log(new Date(), "=========join room start===============");
    if (this.inviteState5ToReturn("joinRoom()")) {
      return;
    };

    wx.showLoading({
      title: '加会中...',
      mask: true
    });

    //  app.data.avdSDK.avdEngine.setVideoCoding(app.data.avdSDK.Enum.VideoCodingType.H264);
    //  app.data.avdSDK.avdEngine.setAudioCoding(app.data.avdSDK.Enum.AudioCodingType.opus);

    console.log(new Date(),"joinRoom(),serverUrl:", app.data.config.serverUrl);
    console.log(new Date(),"joinRoom(),accessToken:", app.data.accessToken);

    app.data.avdSDK.avdEngine.init(app.data.config.serverUrl, app.data.accessToken).then(this.initSuccess).catch(this.initError);
  },

  initSuccess: function() {
    console.log(new Date(), "=====initSuccess====");

    if (this.inviteState5ToReturn("initSuccess()")) {
      return;
    };

    app.data.avdSDK.room = app.data.avdSDK.avdEngine.obtainRoom(this.data.roomId);
    app.data.avdSDK.room.join(app.data.userInfo.webChatId, app.data.userInfo.webChatName, null, null).then(this.joinSuccess).catch(this.joinError);
  },

  initError: function(e) {
    var self = this;
    console.log(new Date(), "=====initError()=====,error:", e);
    
    self.sendRpcInviteFailNotification(101, e.message);

    if (self.inviteState5ToReturn("initError()")) {
       return;
    };

    wx.hideLoading();
    wx.showModal({
      title: '出错提示',
      content: "AVDEngine 初始化失败！" + e.code + ",错误信息：" + e.message,
      showCancel: false,
      success(res) {
        if (res.confirm) {
          self.logout(0);
        }
      }
    })
  },

  joinSuccess: function() {
    console.log(new Date(), "=====joinSuccess=====");
    var self = this;

    self.setData({
       isJoinSucces: true,
    });
   
    if (self.inviteState5ToReturn("joinSuccess()")) {
        if (app.data.callInvite) {
            app.data.callInvite.close("hangUp");
        } 
        self.leaveRoom();
    }else{
        wx.hideLoading();
        console.log(new Date(), "======WWWWW=========joinSuccess()=======inviteState:", self.data.inviteState);
        var url = "../home/home";
        wx.reLaunch({
          url: url
        });
    }
  },

  leaveRoom: function () {
    var self = this;
    var reason = 1; //退会原因
    app.data.avdSDK.room.leave(reason).then(function () {
      console.log("======WWWWWWWWWW===joinSuccess(),inviteState==5 to return, to roomLeave");
      self.logout(0);
    });
  },


  joinError: function(e) {
    console.log(new Date(), "=====joinError=====error:", e);

    var self = this;
    self.sendRpcInviteFailNotification(102, e.message);


    if (self.inviteState5ToReturn("joinError()")) {
       return;
    };

    wx.hideLoading();
   
    wx.showModal({
      title: '出错提示',
      content: "加会失败了! 错误码:" + e.code + ",错误信息：" + e.message,
      showCancel: false,
      success(res) {
        if (res.confirm) {
          self.logout(0);
        }
      }
    })
  },


  //创建房间
  createRoom: function() {
    var self = this;
    console.log(new Date(),"==============createRoom()...");
    var promise = new Promise((resolve, reject) => {
      var topic = 'testingMeeting';
      var userId = app.data.avdSDK.client.selfUser.userId;
      console.log(new Date(),"==============createRoom().userId:",userId);
      api.createRoom(app.data.accessToken, topic, userId,"",true).then(function(res){
        console.log(new Date(),'restApi.createRoom() Success.res:', res)
        resolve(res.room_id);
      }).catch(function(error) {
        console.log(new Date(),"createRoom(),error:", error);
        reject(error);
      })
    });
    return promise;
  },


  //上报客户端参数
  sendAcdSetClientOptions: function(){
    console.log(new Date(),"==============sendAcdSetClientOptions()");

    var self = this;
    var jsonObj = {
      "jsonrpc": "2.0",
      "id": "505",
      "method": "acdSetClientOptions",
      "params": {
         "clientType": 2,
         "clientID": app.data.userInfo.webChatId,
         "options": {
            "dataSource": "北京",
            "backInfo": "",
            "userinfo": "13175645342",
        }
      } 
    }
    
    var jsonStr = JSON.stringify(jsonObj);
    console.log(new Date(), "===call.js,sendAcdSetClientOptions():====jsonStr:", jsonStr);

    self.data.rpcManager.notification(jsonObj).then(function () {
        console.log(new Date(), "rpc notification  acdSetClientOptions  success");
    }).catch(function (error) {
        self.setData({
           tipsMessage: "上报客户端信息时RPC通知(acdSetClientOptions)失败。" + error.message
        })
    })
  },
  
  sendRpcCall: function() {
    var self = this;
    var jsonObj = {
      "jsonrpc": "2.0",
      "id": "500",
      "method": "acdCall",
      "params": {
         "businessCode": app.data.userInfo.params.businessCode,
         "sortValue": app.data.userInfo.params.sortValue
      }
    };

    var jsonStr = JSON.stringify(jsonObj);
    console.log(new Date(),"===call.js,sendRpcCall():====jsonStr:", jsonStr);

    self.data.rpcManager.call(jsonObj).then(function(rpcData) {
       console.log(new Date(),"rpc acdCall成功, 调用返回rpcData :", rpcData);
       self.setTipsMessage(rpcData);
    }).catch(function(error) {
       console.log(new Date(),"RPC acdCall错误，error:", error);
       self.setTipsMessageError(error);
    })
  },

  //客户端加会错误上报，统计报表需要
  sendRpcInviteFailNotification: function (code,message) {
    var self = this;
    var jsonObj = {
      "jsonrpc": "2.0",
      "id": "505",
      "method": "rpcInviteFailNotification",
      "params": {
        "clientid": app.data.userInfo.webChatId,
        "code": code,
        "msg": message
      }
    };

    var jsonStr = JSON.stringify(jsonObj);
    console.log(new Date(), "===call.js,sendRpcInviteFailNotification():====jsonStr:", jsonStr);

    self.data.rpcManager.notification(jsonObj).then(function () {
       console.log(new Date(), "rpc notification  sendRpcInviteFailNotification  success");
    }).catch(function (error) {
      self.setData({
         tipsMessage: "客户端加会错误上报时RPC通知(sendRpcInviteFailNotification)失败。" + error.message
      })
    })
  },


  setTipsMessage: function(data) {
    console.log(new Date(),"=========setTipsMessage()");
    var self = this;
    
    if (data.type == "availableSeat") { //返回可用坐席
      self.setData({
          seatUserId: data.result.userID,
          seatUserName: data.result.username
      })
      console.log(new Date(), "=========setTipsMessage(),availableSeat seatUserId:", data.result.userID, ",seatUserName:", data.result.username);
      self.toCall();
    } else if (data.type == "acdLineLength") { //返回排队人数
      var lineLength = data.result.lineLength;
      self.setData({
        tipsMessage: "当前排队人数" + lineLength + "人，请等待。",
      })
      console.log(new Date(),"=========setTipsMessage(),acdLineLength:", lineLength);
    } else if (data.type == "invalidSeat") { //无可用坐席
      self.setData({
        tipsMessage: "无对应服务坐席，请核实。",
      })
      console.log(new Date(),"=========setTipsMessage(),invalidSeat");
    }
  },

  setTipsMessageError: function(error) {
    var self = this;

    if (error.code == -30000) {
      self.setData({
        tipsMessage: "服务繁忙,稍后再试。[code:-30000]",
      })
      console.log(new Date(), "=========setTipsMessageError(),errorCode:-30000");
    } else if (error.code == 10000){
       self.setData({
         tipsMessage: "网络异常,稍后再试。[code:10001]",
       })
       console.log(new Date(), "=========setTipsMessageError(),errorCode:10001");
    } else {
      self.setData({
        tipsMessage: "RPC错误:" + error.message
      })
      console.log(new Date(), "=========setTipsMessageError(),errorCode:", error.code);
    }
  },


  //取消呼叫
  cancelCall() {
    var self = this;
    var state =  self.data.inviteState;
    console.log(new Date(), "cancelCall(),inviteState:", state);
    if (state == 0) { //邀请状态

      self.setData({
        inviteState: 5,
      });

      var jsonObj = {
        "jsonrpc": "2.0",
        "id": "505",
        "method": "acdCancelCall",
        "params": {
           "businessCode": app.data.userInfo.params.businessCode,
           "sortValue": app.data.userInfo.params.sortValue,
        }
      };

      self.data.rpcManager.notification(jsonObj).then(function () {
        console.log(new Date(), "rpc notification cancelCall success");
        self.logout(0);
      }).catch(function (error) {
        self.setData({
          tipsMessage: "网络异常,稍后再试。[code:10002]"
        })
      })

      return;
    }

    if (state == 1) { //邀请中
      self.setData({
        inviteState: 5,
      });

      app.data.callInvite.cancel().then(function () {
        console.log(new Date(), "callInvite.cancel() success");
        self.logout(0);
      }).catch(function (error) {
        self.setData({
          tipsMessage: "取消呼叫时取消邀请失败。error:" + error.message
        })
      })
      return;
    }

    self.setData({
       inviteState: 5,
    });
    
    if (!self.data.isJoinSucces){
       console.log(new Date(),"===============cancelCall(),joinSucces():false");
       self.logout(0);
    }else{
       console.log(new Date(),"===============cancelCall(),joinSucces():true");
    }
  },

  
  //登出到touch页面
  //reason：登出原因
  logout: function (reason){
    console.log(new Date(), "======WWWWW=========logout()=======");
    app.data.avdSDK.client.logout(reason).then(function () {
      var url = "../touch/touch";
      console.log(new Date(), "======WWWWW=========touch=======");
      wx.reLaunch({
        url: url
      });
    });
  },

  /**
   * 生命周期函数--监听页面初次渲染完成
   */
  onReady: function() {

  },

  /**
   * 生命周期函数--监听页面显示
   */
  onShow: function() {

  },

  /**
   * 生命周期函数--监听页面隐藏
   */
  onHide: function() {

  },

  /**
   * 生命周期函数--监听页面卸载
   */
  onUnload: function() {

  },

  /**
   * 页面相关事件处理函数--监听用户下拉动作
   */
  onPullDownRefresh: function() {

  },

  /**
   * 页面上拉触底事件的处理函数
   */
  onReachBottom: function() {

  },

  /**
   * 用户点击右上角分享
   */
  onShareAppMessage: function() {

  }
})