var api = require('../../restApi/api.js');
var config = require('../../config.js');

//获取应用实例
const app = getApp();
Page({

  /**
   * 页面的初始数据
   */
  data: {
    webChatId: '18626880821',
    webChatName: '王群',
    businessCode:'062',
    sortValue: 8,
   
    webChatIdErrMsg: '',
    errorMessage: ''
  },

  getAccessToken: function () {
    var self = this;
    var promise = new Promise((resolve, reject) => {
      api.getAccessToken(config.accessKey, config.secretKey).then((data) => {
        resolve(data.token);
      }).catch((error) => {
        reject(error);
      });
    });
    return promise;
  },

  sliderDrag:function(e){
    app.data.userInfo.params.sortValue = e.detail.value;
    this.setData({
      sortValue: e.detail.value
    });
  },

  mqttLogin: function () {
    var self = this;

    wx.showLoading({
       title: '路由注册中...',
       mask: true
    });

    var client = app.data.avdSDK.client;
    var keepalive = 10; //连接保活时长,单位秒，可以不填，默认60秒
    client.login(this.data.webChatId, this.data.webChatName,'', keepalive).then(function () {
      wx.hideLoading();
      console.log(new Date(),'路由注册成功');

      var url = "../call/call";
      wx.reLaunch({
        url: url
      });
     
    }).catch(function (e) {
      wx.hideLoading();
      if (e.code ==2){
         e.message = "帐号重复登录";
      } else if (e.code == 1502){
         e.message = "网络异常,连不上路由服务器，请检查网络"
      }
      console.log(new Date(), '路由注册失败,原因:', e.message);
      self.setData({
        errorMessage: "路由注册失败,原因:" + e.message
      })
    });
  },

  //绑定webChatId输入框
  setWebChatId: function (e) {
    var self = this;
    self.setData({
      webChatId: e.detail.value
    });
  },

  //绑定webChatName输入框
  setWebChatName: function (e) {
    var self = this;
    self.setData({
      webChatName: e.detail.value
    });
  },

  //绑定businessCode输入框
  setBusinessCode: function (e) {
    var self = this;
    self.setData({
      businessCode: e.detail.value
    });
  },


  addRoute: function () {
    console.log(new Date(), "touch.js. addRoute()...");
    var self = this;

    wx.showLoading({
      title: '获取访问令牌...',
      mask: true
    });

    self.getAccessToken().then(function (token) {
       wx.hideLoading();
       console.log(new Date(), "accessToken:", token);
       console.log(new Date(), "app.data.config.serverUrl:", app.data.config.serverUrl);
       app.data.accessToken = token;

       app.data.avdSDK.client.setParameters(app.data.config.serverUrl, token);

       self.addRouteHandle();
    }).catch(function (error) {
      wx.hideLoading();
      console.log(new Date(), "getAccessToken(),error:", error);
      if (error.code == 505){
        error.message ="网络异常,连不上路由服务器，请检查网络";
      }
      self.setData({
        errorMessage: '获取访问令牌失败。原因:' + error.message
      })
    });
  },

  addRouteHandle: function(){
    console.log(new Date(), "touch.js. addRouteHandle()...");
    var self  = this;

    var webChatId = self.data.webChatId;
    if (!webChatId) {
      self.setData({
        webChatIdErrMsg: "请输入微信号"
      })
      return;
    } else {
      self.setData({
        webChatIdErrMsg: ""
      })
    }

    var businessCode = self.data.businessCode;
    if (!businessCode) {
      self.setData({
        webChatIdErrMsg: "请输入业务代号"
      })
      return;
    } else {
      self.setData({
        webChatIdErrMsg: ""
      })
    }

    var sortValue = self.data.sortValue;

    app.data.userInfo.webChatId = webChatId;
    app.data.userInfo.webChatName = self.data.webChatName;
    app.data.userInfo.params.businessCode = businessCode;
    app.data.userInfo.params.sortValue = sortValue


    wx.showLoading({
      title: '路由加入中...',
      mask: true
    });

    if (app.data.avdSDK.client.serverURI && app.data.avdSDK.client.accessToken) {
      console.log(new Date(), "============addRouteHandle(),== to mqttLogin...");
      wx.hideLoading();
      self.mqttLogin();
    } else {
      var hideLoadingTimout = 0;
      var intervalServerUrlAndTock = setInterval(function () {
        if (hideLoadingTimout >= 70000) {
          clearInterval(intervalServerUrlAndTock);
          wx.hideLoading();
          console.log(new Date(), '网络异常,连不上路由服务器。');
          self.setData({
            errorMessage: "网络异常,连不上路由服务器，请检查网络！"
          })
        } else {
          hideLoadingTimout = hideLoadingTimout + 50;
          console.log(new Date(), "============addRouteHandle(),=== to intervalServerUrlAndTock...");
          if (app.data.avdSDK.client.serverURI && app.data.avdSDK.client.accessToken) {
            clearInterval(intervalServerUrlAndTock);
            wx.hideLoading();
            self.mqttLogin();
          }
        }

      }, 50);
    }
  },

  toShowBusinessCod: function () {
    var self = this;
    wx.showActionSheet({
      itemList: ['011', '021', '031', '041', '051'],
      success: function (res) {
        if (res.tapIndex == 0) {
          self.setData({
            businessCode: '011'
          });
        } else if (res.tapIndex == 1){
          self.setData({
            businessCode: '021'
          });
        } else if (res.tapIndex == 2) {
          self.setData({
            businessCode: '031'
          });
        } else if (res.tapIndex == 3) {
          self.setData({
            businessCode: '041'
          });
        } else if (res.tapIndex == 4) {
          self.setData({
            businessCode: '051'
          });
        }
      },
      fail: function (res) {
        console.log(res.errMsg)
      }
    })
  },



  /**
   * 生命周期函数--监听页面加载
   */
  onLoad: function (options) {
    var self = this;
    
    app.data.avdSDK.avdEngine = new app.data.avdSDK.avdSDKInstance.AVDEngine();
    app.data.avdSDK.client = new app.data.avdSDK.avdSDKInstance.Client();
    app.data.avdSDK.signalClient = app.data.avdSDK.client;
  },



  /**
   * 生命周期函数--监听页面初次渲染完成
   */
  onReady: function () {

  },

  /**
   * 生命周期函数--监听页面显示
   */
  onShow: function () {

  },

  /**
   * 生命周期函数--监听页面隐藏
   */
  onHide: function () {

  },

  /**
   * 生命周期函数--监听页面卸载
   */
  onUnload: function () {

  },

  /**
   * 页面相关事件处理函数--监听用户下拉动作
   */
  onPullDownRefresh: function () {

  },

  /**
   * 页面上拉触底事件的处理函数
   */
  onReachBottom: function () {

  },

  /**
   * 用户点击右上角分享
   */
  onShareAppMessage: function () {

  }
})